%%% Monte Carlo Dynamic Classifier tool - MCDC tool
% http://www.ismvideo.org/NDE0UP/MCDCtool/en/index.html

% Copyright (C) 2014-2015 Gareth W. Peters, Tomoko Matsui, Francois Septier, and Akira Tamamori

% This program is free software; you can redistribute it and/or modify it under the terms
% of the GNU General Public License as published by the Free Software Foundation; either
% version 2 of the License, or (at your option) any later version.

% This program is distributed in the hope that it will be useful, but WITHOUT ANY
% WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
% PARTICULAR PURPOSE.  See the GNU General Public License for more details.

% You should have received a copy of the GNU General Public License along with this
% program; if not, see <http://www.gnu.org/licenses/>.

function plotLorenz_Obs (matfile)

    [~, in, out] = MCDCMatFile.Load(matfile);

    load('logs/LorenzModelPMCMC2_Data.mat');

    J = out.j;
    T = size(out.YE, 2);
    dim = size(out.YE, 3);
    ts = 1 : T;

    YE = out.YEAt(1 : J);
    YEMean = squeeze(mean(YE(:, ts, :), 1));
    YEStd = squeeze(std(YE(:, ts, :), 1));

    for d = 1 : dim
        fig_num = 10 + d;
        figure(fig_num); clf;
        set(gcf, 'DefaultAxesFontSize', 30);
        set(gcf, 'DefaultTextFontSize', 30);
        position = get(gcf, 'Position');
        position(3) = 1000;
        position(4) = 400;
        set(gcf, 'Position', position);

        paper_pos = get(gcf, 'PaperPosition');
        set(gcf, 'PaperPosition', [paper_pos(1) paper_pos(2) 64 26]);
        plot(ts, in.z(d, ts), 'Color', 'red');
        hold on;
        if (d == 1)
            ylim([-15 80]);
        elseif (d == 2)
            ylim([-100 900]);
        elseif (d == dim)
            ylim([-110 1200]);
        end
        xlabel('Time');
        plot(ts, YEMean(:, d), 'Color', 'blue');
        plot(ts, YEMean(:, d) + YEStd(:, d), 'Color', 'green', 'LineStyle', '--');
        plot(ts, YEMean(:, d) - YEStd(:, d), 'Color', 'green', 'LineStyle', '--');
        legend({'Obseravtion', 'Mean', '+\sigma (standard dev.)', '-\sigma (standard dev.)'}, 'Fontsize', 30);
        hold off;

        file_name = sprintf('plots/LorenzModelPMCMC2_Obs_dim%d.png', d);
        saveas(gcf, file_name);

        file_name = sprintf('plots/LorenzModelPMCMC2_Obs_dim%d.eps', d);
        print('-r0','-depsc2', file_name);
    end
end
